/***********************************************************
 *
 * d_Backlight.c
 * SYSTART GmbH
 * SAM, 22.06.2016
 *
 ***********************************************************/

/***********************************************************
 * Includes
 ***********************************************************/
#include "driver/d_Backlight.h"

/***********************************************************
 * Global Variables
 ***********************************************************/
TIM_OCInitTypeDef  TIM_OCInitStruct;
GPIO_InitTypeDef GPIO_InitStruct;

/***********************************************************
 * Function definitions
 ***********************************************************/
void Backlight_init ( void )
{
	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStruct;

	// Activate clocks
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOH, ENABLE );
	RCC_APB1PeriphClockCmd( RCC_APB1Periph_TIM12, ENABLE );

	// Init of timer
	TIM_TimeBaseStruct.TIM_Period = 9000 - 1;
	TIM_TimeBaseStruct.TIM_Prescaler = 0;
	TIM_TimeBaseStruct.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStruct.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStruct.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit( TIM12, &TIM_TimeBaseStruct );
}

void Backlight_configPWM ( uint16_t dutycycle )
{
	uint16_t pulse = 0;
	dutycycle = (uint16_t) (100*(pow(((double)dutycycle/100), 2.2)) + 0.5);

	pulse = 9000 / 100 * dutycycle;

	if ( pulse < 450 )
		TIM_OCInitStruct.TIM_Pulse = 450;
	else if ( pulse > 8550 )
		TIM_OCInitStruct.TIM_Pulse = 8550;
	else
		TIM_OCInitStruct.TIM_Pulse = pulse;
}

void Backlight_ON ( void )
{
	GPIO_ResetBits( GPIOH, GPIO_Pin_6 );

	// Init of Pins
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_High_Speed;
	GPIO_Init( GPIOH, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOH, GPIO_PinSource6, GPIO_AF_TIM12 );

	// Init of PWM
	TIM_OCInitStruct.TIM_OCMode = TIM_OCMode_PWM1;
	TIM_OCInitStruct.TIM_OutputState = TIM_OutputState_Enable;
	if ( TIM_OCInitStruct.TIM_Pulse == 0 )
		TIM_OCInitStruct.TIM_Pulse = 4500;
	TIM_OCInitStruct.TIM_OCPolarity = TIM_OCPolarity_High;
	TIM_OCInitStruct.TIM_OCIdleState = TIM_OCIdleState_Reset;
	TIM_OC1Init( TIM12, &TIM_OCInitStruct );
	TIM_OC1PreloadConfig( TIM12, TIM_OCPreload_Enable );
	TIM_ARRPreloadConfig( TIM12, ENABLE );
	TIM_CtrlPWMOutputs( TIM12, ENABLE );
	TIM_Cmd( TIM12, ENABLE );
}

void Backlight_OFF ( void )
{
	TIM_Cmd( TIM12, DISABLE );
	TIM_SetCounter( TIM12, 0 );

	GPIO_ResetBits( GPIOH, GPIO_Pin_6 );

	// Init of Pins
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6;
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_Speed = GPIO_High_Speed;
	GPIO_Init( GPIOH, &GPIO_InitStruct );
}
