/***********************************************************
 *
 * d_SDRAM_FLASH.c
 * SYSTART GmbH
 * SAM, 23.06.2017
 * RAT, 2018 (NOR_Flash functions)
 *
 ***********************************************************/

/***********************************************************
 * Includes
 ***********************************************************/
#include "driver\d_SDRAM_FLASH.h"
#include "stm32f4xx_fmc.h"

/***********************************************************
 * Declaration internal functions
 ***********************************************************/
void SDRAM_NOR_FLASH_GPIOs_init ( void );
void SDRAM_Initialization_Sequence ( void );


/***********************************************************
 * Declaration internal macros
 ***********************************************************/
#define ADDR_SHIFT(A) (NOR_FLASH_START_ADDR + (2 * (A)))

/***********************************************************
 * Function definitions
 ***********************************************************/
void SDRAM_FLASH_init ( void )
{
	FMC_SDRAMTimingInitTypeDef FMC_SDRAMTimingStruct;
	FMC_SDRAMInitTypeDef FMC_SDRAMInitStruct;

	// Initialize GPIOs for SDRAM
	SDRAM_NOR_FLASH_GPIOs_init();

	// Activate clock
	RCC_AHB3PeriphClockCmd( RCC_AHB3Periph_FMC, ENABLE );

	// Initialize SDRAM
	FMC_SDRAMTimingStruct.FMC_LoadToActiveDelay = 2;
	FMC_SDRAMTimingStruct.FMC_ExitSelfRefreshDelay = 6;
	FMC_SDRAMTimingStruct.FMC_SelfRefreshTime = 4;
	FMC_SDRAMTimingStruct.FMC_RowCycleDelay = 6;
	FMC_SDRAMTimingStruct.FMC_WriteRecoveryTime = 2;
	FMC_SDRAMTimingStruct.FMC_RPDelay = 2;
	FMC_SDRAMTimingStruct.FMC_RCDDelay = 2;

	FMC_SDRAMInitStruct.FMC_Bank = FMC_Bank1_SDRAM;
	FMC_SDRAMInitStruct.FMC_ColumnBitsNumber = FMC_ColumnBits_Number_8b;
	FMC_SDRAMInitStruct.FMC_RowBitsNumber = FMC_RowBits_Number_12b;
	FMC_SDRAMInitStruct.FMC_SDMemoryDataWidth = FMC_SDMemory_Width_32b;
	FMC_SDRAMInitStruct.FMC_InternalBankNumber = FMC_InternalBank_Number_4;
	FMC_SDRAMInitStruct.FMC_CASLatency = FMC_CAS_Latency_3;
	FMC_SDRAMInitStruct.FMC_WriteProtection = FMC_Write_Protection_Disable;
	FMC_SDRAMInitStruct.FMC_SDClockPeriod = FMC_SDClock_Period_2;
	FMC_SDRAMInitStruct.FMC_ReadBurst = FMC_Read_Burst_Enable;
	FMC_SDRAMInitStruct.FMC_ReadPipeDelay = FMC_ReadPipe_Delay_0;
	FMC_SDRAMInitStruct.FMC_SDRAMTimingStruct = &FMC_SDRAMTimingStruct;

	FMC_SDRAMInit( &FMC_SDRAMInitStruct );

	SDRAM_Initialization_Sequence();

}

void SDRAM_Write_8b ( uint32_t adr, uint8_t data )
{
	*(uint8_t*)( SDRAM_START_ADDR + adr ) = data;
}

void SDRAM_Write_16b ( uint32_t adr, uint16_t data )
{
	*(uint16_t*)( SDRAM_START_ADDR + adr ) = data;
}

void SDRAM_Write_32b ( uint32_t adr, uint32_t data )
{
	*(uint32_t*)( SDRAM_START_ADDR + adr ) = data;
}

uint8_t SDRAM_Read_8b ( uint32_t adr )
{
	uint8_t ret = 0;

	ret = *(__IO uint8_t*)( SDRAM_START_ADDR + adr );

	return ret;
}

uint16_t SDRAM_Read_16b ( uint32_t adr )
{
	uint16_t ret = 0;

	ret = *(__IO uint16_t*)( SDRAM_START_ADDR + adr );

	return ret;
}

uint32_t SDRAM_Read_32b ( uint32_t adr )
{
	uint32_t ret = 0;

	ret = *(__IO uint32_t*)( SDRAM_START_ADDR + adr );

	return ret;
}


/***********************************************************
 * Definitions internal functions
 ***********************************************************/
void SDRAM_NOR_FLASH_GPIOs_init ( void )
{
	GPIO_InitTypeDef GPIO_InitStruct;

	// Activate clocks
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOD, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOE, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOF, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOG, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOH, ENABLE );
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOI, ENABLE );

	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_100MHz;

	// PD0  = FMC_D2, PD1  = FMC_D3
	// PD4 = FMC_NOE (NOR_G), PD5 = FMC_NWE (NOR_W), PD7 = FMC_NE1 (NOR_E)
	// PD8  = FMC_D13, PD9  = FMC_D14, PD10 = FMC_D15
	// PD11 = FMC_A16, PD12 = FMC_A17, PD13 = FMC_A18
	// PD14 = FMC_D0,  PD15 = FMC_D1
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_7 | 	//Pin 4, 5 and 7 only for NOR Flash required
			GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 |
			GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init( GPIOD, &GPIO_InitStruct );

	// PD6 = FMC_NWAIT (NOR_RB) mit PullUp da Opendrain, danach Pull-Up wieder zuruecksetzen
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6;
	GPIO_Init( GPIOD, &GPIO_InitStruct );

	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;

	GPIO_PinAFConfig( GPIOD, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource6, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource7, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource8, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource9, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource10, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource11, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource12, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource13, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource14, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOD, GPIO_PinSource15, GPIO_AF_FMC );

	// PE0  = FMC_NBL0, PE1  = FMC_NBL1
	// PE3 = FMC_A19, PE4 = FMC_A20, PE5 = FMC_A21, PE6 = FMC_A22
	// PE7  = FMC_D4, PE8  = FMC_D5, PE9  = FMC_D6, PE10 = FMC_D7,
	// PE11 = FMC_D8, PE12 = FMC_D9, PE13 = FMC_D10, PE14 = FMC_D11,
	// PE15 = FMC_D12
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_3 | GPIO_Pin_4 |
			GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 |
			GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 |
			GPIO_Pin_15;
	GPIO_Init( GPIOE, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOE, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource6, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource7, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource8, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource9, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource10, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource11, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource12, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource13, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource14, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOE, GPIO_PinSource15, GPIO_AF_FMC );

	// PF0  = FMC_A0, PF1  = FMC_A1, PF2  = FMC_A2, PF3  = FMC_A3
	// PF4  = FMC_A4, PF5  = FMC_A5
	// PF11 = SDNRAS, PF12 = FMC_A6, PF13 = FMC_A7, PF14 = FMC_A8,
	// PF15 = FMC_A9
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
			GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 |
			GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init( GPIOF, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOF, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource2, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource11, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource12, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource13, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource14, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOF, GPIO_PinSource15, GPIO_AF_FMC );

	// PG0  = FMC_A10, PG1  = FMC_A11, PG2 = FMC_A12, PG3 = FMC_A13
	// PG4  = FMC_A14 (FMC_BA0), PG5  = FMC_A15 (FMC_BA1)
	// PG8  = SDCLK, PG15 = SDNCAS
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
			GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_8 | GPIO_Pin_15;
	GPIO_Init( GPIOG, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOG, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource2, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource8, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOG, GPIO_PinSource15, GPIO_AF_FMC );

	// PH2  = SDNCKE, PH3  = SDNE0, PH5  = SDNWE
	// PH8  = FMC_D16, PH9  = FMC_D17, PH10 = FMC_D18, PH11 = FMC_D19
	// PH12 = FMC_D20, PH13 = FMC_D21, PH14 = FMC_D22, PH15 = FMC_D23
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_5 | GPIO_Pin_8 |
			GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12 | GPIO_Pin_13 |
			GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init( GPIOH, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOH, GPIO_PinSource2, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource8, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource9, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource10, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource11, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource12, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource13, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource14, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOH, GPIO_PinSource15, GPIO_AF_FMC );

	// PI0  = FMC_D24, PI1  = FMC_D25, PI2  = FMC_D26, PI3  = FMC_D27,
	// PI4  = FMC_NBL2, PI5  = FMC_NBL3, PI6  = FMC_D28, PI7  = FMC_D29
	// PI9  = FMC_D30, PI10 = FMC_D31
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 |
			GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_9 |
			GPIO_Pin_10;
	GPIO_Init( GPIOI, &GPIO_InitStruct );

	GPIO_PinAFConfig( GPIOI, GPIO_PinSource0, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource1, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource2, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource3, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource4, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource5, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource6, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource7, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource9, GPIO_AF_FMC );
	GPIO_PinAFConfig( GPIOI, GPIO_PinSource10, GPIO_AF_FMC );
}

void SDRAM_Initialization_Sequence( void )
{
	FMC_SDRAMCommandTypeDef FMC_SDRAMCommand;
	uint32_t tmpmrd = 0;
	uint32_t count = 0;

	// Step1: Configure a clock configuration enable command
	FMC_SDRAMCommand.FMC_CommandMode = FMC_Command_Mode_CLK_Enabled;
	FMC_SDRAMCommand.FMC_CommandTarget = FMC_Command_Target_bank1;
	FMC_SDRAMCommand.FMC_AutoRefreshNumber = 1;
	FMC_SDRAMCommand.FMC_ModeRegisterDefinition = 0;

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	// Step2: 100us minimum delay
	for ( count = 0; count < 180000; ++count )
		asm("nop");

	// Step3: Configure PALL command
	FMC_SDRAMCommand.FMC_CommandMode = FMC_Command_Mode_PALL;
	FMC_SDRAMCommand.FMC_CommandTarget = FMC_Command_Target_bank1;
	FMC_SDRAMCommand.FMC_AutoRefreshNumber = 1;
	FMC_SDRAMCommand.FMC_ModeRegisterDefinition = 0;

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	// Step4: Configure Auto Refresh command
	FMC_SDRAMCommand.FMC_CommandMode = FMC_Command_Mode_AutoRefresh;
	FMC_SDRAMCommand.FMC_CommandTarget = FMC_Command_Target_bank1;
	FMC_SDRAMCommand.FMC_AutoRefreshNumber = 8;
	FMC_SDRAMCommand.FMC_ModeRegisterDefinition = 0;

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	// Step5: Programm external memory mode register
	tmpmrd = (uint32_t) SDRAM_MODEREG_BURST_LENGTH_1 |
            			SDRAM_MODEREG_BURST_TYPE_SEQUENTIAL   |
						SDRAM_MODEREG_CAS_LATENCY_3           |
						SDRAM_MODEREG_OPERATING_MODE_STANDARD |
						SDRAM_MODEREG_WRITEBURST_MODE_SINGLE;
	FMC_SDRAMCommand.FMC_CommandMode = FMC_Command_Mode_LoadMode;
	FMC_SDRAMCommand.FMC_CommandTarget = FMC_Command_Target_bank1;
	FMC_SDRAMCommand.FMC_AutoRefreshNumber = 1;
	FMC_SDRAMCommand.FMC_ModeRegisterDefinition = tmpmrd;

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
	FMC_SDRAMCmdConfig( &FMC_SDRAMCommand );

	FMC_SetRefreshCount( SDRAM_REFRESH_COUNT );

	while( FMC_GetFlagStatus( FMC_Bank1_SDRAM, FMC_FLAG_Busy ) != RESET );
}
