/**
  ******************************************************************************
  * @package Digital Picture Frame
  * @author  EBS-SYSTART GmbH
  * @version V1.0
  * @date    06-July-2021
  * @brief   Implements a basic digital picture frame which loads, decodes and
			 displays JPEG files from an inserted SD-card.
  ******************************************************************************
*/

/***********************************************************
 * Includes
 ***********************************************************/
#include "stm32f4xx.h"
/* Driver */
#include "driver/d_timer.h"
#include "driver/d_Backlight.h"
#include "driver/d_Display_DMA2D.h"
#include "driver/d_SDIO.h"
 /* Applications */
#include "application/paintJPEG.h"
#include "mymalloc.h"

#include <stdio.h>

#if 0
#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */
#endif /* For Debugging */


/***********************************************************
 * Private Function Prototypes
 ***********************************************************/
void HW_Init(void);
void gpio_init(void);

/*****************************************************************
 *
 * Function: int main(void)
 *
 * Summary: Entry point of display control application
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date:
 *
 *****************************************************************/
int main(void) {

	/* Initialize the peripherals */
	HW_Init();
	display_init();
	initMemBlock();
	disk_initialize(0);

	// 20 jpges with file path shorter than 50 characters (including filename).
	//If longer file paths are expected the value most also be changed in "tm_stm32f4_fatfs.c" in the "scan_files"-function
	char jpeg_filename_array[20][50];
	int jpeg_count = SDIO_search_for_jpeg(&jpeg_filename_array[0][0]);
	for(;;){
		for(int i=0; i<jpeg_count; i++){
			char jpeg_filename[50];
			strcpy(jpeg_filename, (const char*)(jpeg_filename_array+i));
			paint_JPEG_file(jpeg_filename);
		}
	}
	return 0;
}

/***********************************************************
 * Private Functions
 ***********************************************************/

void WWDG_IRQHandler(){
	while(1){

	}
}
void TIM8_UP_TIM13_IRQHandler(){
	while(1){

	}
}
void LCD_TFT_IRQHandler(){
	while(1){

	}
}
void LCD_TFT_1_IRQHandler(){
	while(1){

	}
}

/*****************************************************************
 *
 * Function:void HW_Init(void)
 *
 * Summary: Various Hardware initializations
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
void HW_Init(void) {
	SystemInit();

	timer_init();

	Backlight_init();
	Backlight_configPWM(95);
	Backlight_ON();
}

/*****************************************************************
 *
 * Function: void gpio_init(void)
 *
 * Summary: Configure GPIOs for SD-card detect pin
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 30052019
 *
 *****************************************************************/
void gpio_init(void) {
	GPIO_InitTypeDef GPIO_InitStruct;

	// Activating clocks
	RCC_AHB1PeriphClockCmd( RCC_AHB1Periph_GPIOG, ENABLE );

	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_2MHz;

	//PG10 = SDIO_CD
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init( GPIOG, &GPIO_InitStruct);
}

#if 0 //For Debugging
PUTCHAR_PROTOTYPE
{
  USART_SendData(UART[0].UART, (uint8_t) ch);

  /* Loop until the end of transmission */
  while (USART_GetFlagStatus(UART[0].UART, USART_FLAG_TC) == RESET)
  {}

  return ch;
}
#endif
