#include<stdio.h>
#include<stddef.h>
#include "driver/d_SDRAM_FLASH.h"
#include "mymalloc.h"

/*Pointing to the main block of memory which is initially free (no storage allocation yet)*/
struct block *freeList= (void*)memory;

/* Fill the used memory block of SDRAM with zeros */
void initMemBlock(){
	for(int i=0; i<8000000; i+=4){
		SDRAM_Write_32b ((memory-0xC0000000)+i, 0);
	}
}

/*Initializing the block of memory*/
void initialize(){
	freeList->size=8000000-sizeof(struct block);
	freeList->free=1;
	freeList->next=NULL;
}


/*Making way for a new block allocation by splitting a free block -- (Assume first fit algorithm)*/
void split(struct block *fitting_slot, size_t size){
	struct block *new=(void*)((void*)fitting_slot+size+sizeof(struct block));
	new->size=(fitting_slot->size)-size-sizeof(struct block);
	new->free=1;
	new->next=fitting_slot->next;
	fitting_slot->size=size;
	fitting_slot->free=0;
	fitting_slot->next=new;
}


/*Function MyMalloc(malloc)*/
void *MyMalloc(size_t noOfBytes){
	struct block *curr,*prev;	//We need metadata block pointers to traverse through the freeList.
	void *result;	//This result pointer is to return the starting address of the allocated chunk of memory.

	//This condition will initialize the memory if it is not initialized at the beginning.
	//This set of statements will be executed only if the size of the first metadata block is not set yet. That means, if the memory is still not initialized.
	if(!(freeList->size)){
		initialize();
	}

	curr=freeList;	//Now we make the temporary pointer curr to point to the start of the metadata block list.

	//If this condition is met, the metadata block we checked cannot be used for the allocation.
	//So we execute the following statements. That is you go on checking one metadata block at a time.
	while((((curr->size)<noOfBytes)||((curr->free)==0))&&(curr->next!=NULL)){
		prev=curr;
		curr=curr->next;
	}

	//If this condition is met, the metadata block we checked refers to a chunk of memory that exactly fits the required size.
	//So set the free flag to 0, indicating that it is allocated. Then return the starting address of the block of allocated memory.
	if((curr->size)==noOfBytes){
		curr->free=0;
		result=(void*)(++curr);
		return result;
	}
	//If this condition is met, the metadata block we checked refers to a chunk of memory that is of size greater than what is required.
	//So invoke the split() function to allocate only the block which is required and then return the starting address of the block of allocated memory.
	else if((curr->size)>(noOfBytes+sizeof(struct block))){
		split(curr,noOfBytes);
		result=(void*)(++curr);
		return result;
	}
	//Else it means that there is no sufficient memory to allocate so you should return a NULL pointer.
	else{
		result=NULL;
		return result;
	}
}


/*This is to merge the consecutive free blocks by removing the metadata block in the middle. This will save space.*/
void merge(){
	struct block *curr,*prev;
	curr=freeList;
	while((curr->next)!=NULL){
		if((curr->free) && (curr->next->free)){
			curr->size+=(curr->next->size)+sizeof(struct block);
			curr->next=curr->next->next;
		}
		prev=curr;
		if(curr->next != NULL){
			curr=curr->next;
		}
	}
}


/*Function MyFree(free)*/
void MyFree(void* ptr){
    //Here we check whether the address to which the pointer given as an argument to the function actually lies within the address range of the memory array that we used for the purpose.
	//If yes,we simply set the free flag in the metadata block to 1 indicating that it is free and scan through and merge the consecutive blocks that are free, if any.
	if(((void*)memory<=ptr)&&(ptr<=(void*)(memory+8000000))){
		struct block* curr=ptr;
		--curr;
		curr->free=1;
		merge();
	}
}



